<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Controller\Change;

use FiloBlu\ExtAmastyGeoipRedirect\Helper\Data;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\Response\RedirectInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Stdlib\Cookie\CookieSizeLimitReachedException;
use Magento\Framework\Stdlib\Cookie\FailureToSendException;
use Magento\Framework\View\Result\Page;
use Magento\Framework\View\Result\PageFactory;
use Magento\Store\Api\StoreCookieManagerInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\UrlRewrite\Model\UrlFinderInterface;


/**
 * Responsible for loading page content.
 *
 * This is a basic controller that only loads the corresponding layout file. It may duplicate other such
 * controllers, and thus it is considered tech debt. This code duplication will be resolved in future releases.
 */
class Country extends Action
{

    /**
     * @var PageFactory
     */
    protected $resultJsonFactory;

    /**
     * @var Data
     */
    protected $_helper;

    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var Session
     */
    protected $customerSession;

    /**
     * @var StoreCookieManagerInterface
     */
    protected $storeCookieManager;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * Country constructor.
     * @param Context $context
     * @param JsonFactory $resultJsonFactory
     * @param Data $helper
     * @param StoreManagerInterface $storeManager
     * @param Session $customerSession
     * @param StoreCookieManagerInterface $storeCookieManager
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        Context $context,
        JsonFactory $resultJsonFactory,
        Data $helper,
        StoreManagerInterface $storeManager,
        Session $customerSession,
        StoreCookieManagerInterface $storeCookieManager,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->_helper = $helper;
        $this->_storeManager = $storeManager;
        $this->customerSession = $customerSession;
        $this->storeCookieManager = $storeCookieManager;
        $this->scopeConfig = $scopeConfig;

        parent::__construct($context);
    }

    /**
     * @return ResponseInterface|Json|ResultInterface|Page|null
     * @throws InputException
     * @throws LocalizedException
     * @throws NoSuchEntityException
     * @throws CookieSizeLimitReachedException
     * @throws FailureToSendException
     */
    public function execute()
    {
        if (!$this->getRequest()->isPost()) {
            return null;
        }

        if (!$this->getRequest()->getPost('store_id')) {
            return null;
        }

        if (!$this->getRequest()->getPost('country_id')) {
            return null;
        }

        $storeId = $this->getRequest()->getPost('store_id');
        $countryId = $this->getRequest()->getPost('country_id');
        $store = $this->_storeManager->getStore($storeId);
        $websiteId = $store->getWebsiteId();
        $stores = $this->_storeManager->getWebsite($websiteId)->getStores();
        $actualUrl = null;
        $active_languages = [];

        if ($this->scopeConfig->getValue('amgeoipredirect/country_url/enable_url')) {
            $urlMapping = unserialize(
                $this->scopeConfig->getValue(
                    'amgeoipredirect/country_url/url_mapping',
                    ScopeInterface::SCOPE_STORES,
                    $storeId
                )
            );
            foreach ($urlMapping as $countries => $url) {
                if (strpos($countries, $countryId) !== false) {
                    $staticRedirectUrl = [];
                    $staticRedirectUrl[] = [
                        'name'       => $countryId,
                        'store_id'   => $storeId,
                        'url'        => $url,
                        'country_id' => $countryId
                    ];
                    $resultJson = $this->resultJsonFactory->create();
                    $resultJson->setData($staticRedirectUrl);

                    return $resultJson;
                }
            }
        }

        if (!$this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_to_homepage')) {
            /** @var  \Magento\Framework\App\Response\RedirectInterface $redirect */
            $redirect = ObjectManager::getInstance()->get(RedirectInterface::class);
            /** @var  \Magento\UrlRewrite\Model\UrlFinderInterface $urlFinder */
            $urlFinder = ObjectManager::getInstance()->get(UrlFinderInterface::class);

            $refererUrl = $redirect->getRefererUrl();

            foreach ($this->_storeManager->getStores() as $fromStore) {
                $baseUrl = $fromStore->getBaseUrl();
                if (strncasecmp($baseUrl, $refererUrl, strlen($baseUrl)) === 0) {
                    $actualUrl = $urlFinder->findOneByData(
                        [
                            'request_path' => str_replace($baseUrl, '', $refererUrl),
                            'store_id'     => $fromStore->getId()
                        ]
                    );

                    break;
                }
            }

            foreach ($stores as $language) {
                if ($language->isActive()) {
                    $active_languages[] = $language->getId();
                }
            }
        }


        $languages = [];

        foreach ($stores as $language) {
            if ($language->isActive()) {
                $newUrl = $language->getBaseUrl();

                if ($actualUrl) {
                    $destUrl = $urlFinder->findOneByData(
                        [
                            'store_id'         => $language->getId(),
                            'entity_id'        => $actualUrl->getEntityId(),
                            'entity_type'      => $actualUrl->getEntityType(),
                            'is_autogenerated' => 1
                        ]
                    );

                    if (count($active_languages) === 1) {
                        $newUrl = $language->getBaseUrl() . $destUrl->getRequestPath();
                    }
                }

                $languages[] = [
                    'name'       => $language->getName(),
                    'store_id'   => $language->getId(),
                    'url'        => $newUrl,
                    'country_id' => $countryId
                ];
            }
        }

        if (count($languages) === 1) {
            $this->_helper->setCountryCookie($countryId);
            $this->_helper->setStoreCookie($storeId);
            $this->storeCookieManager->setStoreCookie($store);
        }

        $resultJson = $this->resultJsonFactory->create();
        $resultJson->setData($languages);

        return $resultJson;
    }
}
